@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
-    Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -120,10 +120,9 @@ USA.
 		   (keyword-constructor make-keyparser-fragment)
 		   (conc-name keyparser-fragment/)
 		   (print-procedure
-		    (standard-unparser-method 'KEYPARSER-FRAGMENT
-		      (lambda (fragment port)
-			(write-char #\space port)
-			(write (keyparser-fragment/keyword fragment) port)))))
+		    (standard-print-method 'KEYPARSER-FRAGMENT
+		      (lambda (fragment)
+			(list (keyparser-fragment/keyword fragment))))))
   ;; Keyword that introduces the structure.
   (keyword #f read-only #t)
 
@@ -326,10 +325,9 @@ See \\[complete-keyword]."
 (define-structure (keyparser-stack-entry
 		   (conc-name keyparser-stack-entry/)
 		   (print-procedure
-		    (standard-unparser-method 'KEYPARSER-STACK-ENTRY
-		      (lambda (entry port)
-			(write-char #\space port)
-			(write (keyparser-stack-entry/keyword entry) port)))))
+		    (standard-print-method 'KEYPARSER-STACK-ENTRY
+		      (lambda (entry)
+			(list (keyparser-stack-entry/keyword entry))))))
   (pattern #f read-only #t)
   (index #f read-only #t)
   (start #f read-only #t))
