@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -30,6 +31,7 @@ USA.
 
 extern Tchannel OS_tty_input_channel (void);
 extern Tchannel OS_tty_output_channel (void);
+extern Tchannel OS_tty_error_channel (void);
 extern unsigned int OS_tty_x_size (void);
 extern unsigned int OS_tty_y_size (void);
 extern const char * OS_tty_command_beep (void);
